@@ -3,7 +3,7 @@ package Net::DNS::RR;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: RR.pm 1903 2023-03-06 08:51:09Z willem $)[2];
+our $VERSION = (qw$Id: RR.pm 1990 2024-09-18 13:16:07Z willem $)[2];
 
 
 =head1 NAME
@@ -51,14 +51,14 @@ you will get an error message and execution will be terminated.
 
 sub new {
 	my ( $class, @list ) = @_;
-	return eval {
+	my $rr = eval {
 		local $SIG{__DIE__};
 		scalar @list > 1 ? &_new_hash : &_new_string;
-	} || do {
-		my @param = map { defined($_) ? split /\s+/ : 'undef' } @list;
-		my $stmnt = substr "$class->new( @param )", 0, 80;
-		croak "${@}in $stmnt\n";
 	};
+	return $rr if $rr;
+	my @param = map { defined($_) ? split /\s+/ : 'undef' } @list;
+	my $stmnt = substr "$class->new( @param )", 0, 80;
+	croak "${@}in $stmnt\n";
 }
 
 
@@ -89,11 +89,11 @@ my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]+/;		# NB: *not* \s (matches
 
 sub _new_string {
 	my ( $base, $string ) = @_;
-	local $_ = $string;
-	croak 'argument absent or undefined' unless defined $_;
-	croak 'non-scalar argument' if ref $_;
+	die 'argument absent or undefined' unless defined $string;
+	die 'non-scalar argument' if ref $string;
 
 	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
+	local $_ = $string;
 	s/\\\\/\\092/g;						# disguise escaped escape
 	s/\\"/\\034/g;						# disguise escaped quote
 	s/\\\(/\\040/g;						# disguise escaped bracket
@@ -101,7 +101,7 @@ sub _new_string {
 	s/\\;/\\059/g;						# disguise escaped semicolon
 	my ( $owner, @token ) = grep { defined && length } split /$PARSE_REGEX/o;
 
-	croak 'unable to parse RR string' unless scalar @token;
+	die 'unable to parse RR string' unless scalar @token;
 	my $t1 = $token[0];
 	my $t2 = $token[1];
 
@@ -130,7 +130,7 @@ sub _new_string {
 		shift @token;					# RFC3597 hexadecimal format
 		my $rdlen = shift(@token) || 0;
 		my $rdata = pack 'H*', join( '', @token );
-		croak 'length and hexadecimal data inconsistent' unless $rdlen == length $rdata;
+		die 'length and hexadecimal data inconsistent' unless $rdlen == length $rdata;
 		$self->rdata($rdata);				# unpack RDATA
 	} else {
 		$self->_parse_rdata(@token);			# parse arguments
@@ -234,9 +234,11 @@ sub decode {
 	my $next = $index + $self->{rdlength};
 	die 'corrupt wire-format data' if length $$data < $next;
 
-	$self->{offset} = $offset || 0;
-	$self->_decode_rdata( $data, $index, @opaque ) if $next > $index or $self->type eq 'OPT';
-	delete $self->{offset};
+	if ( $next > $index or $self->type eq 'OPT' ) {
+		local $self->{offset} = $offset;
+		eval { $self->_decode_rdata( $data, $index, @opaque ) };
+		warn $@ if $@;
+	}
 
 	return wantarray ? ( $self, $next ) : $self;
 }
@@ -324,18 +326,18 @@ sub string {
 	my @ttl	 = grep {defined} $self->{ttl};
 	my @core = ( $name, @ttl, $self->class, $self->type );
 
+	local $SIG{__DIE__};
 	my $empty = $self->_empty;
 	my @rdata = $empty ? () : eval { $self->_format_rdata };
 	carp $@ if $@;
 
-	my $tab = length($name) < 72 ? "\t" : ' ';
-	$self->_annotation('no data') if $empty;
-
+	my $tab	 = length($name) < 72 ? "\t" : ' ';
 	my @line = _wrap( join( $tab, @core, '(' ), @rdata, ')' );
 
 	my $last = pop(@line);					# last or only line
 	$last = join $tab, @core, "@rdata" unless scalar(@line);
 
+	$self->_annotation('no data') if $empty;
 	return join "\n\t", @line, _wrap( $last, map {"; $_"} $self->_annotation );
 }
 
@@ -526,7 +528,9 @@ sub dump {				## print internal data structure
 }
 
 sub rdatastr {				## historical RR subtype method
-	return &rdstring;					# uncoverable pod
+	my $self = shift;					# uncoverable pod
+	$self->_deprecate('prefer $rr->rdstring()');
+	return $self->rdstring;
 }
 
 
@@ -559,6 +563,7 @@ Returns a string representation of the RR-specific data.
 
 sub rdstring {
 	my $self = shift;
+	local $SIG{__DIE__};
 
 	my @rdata = $self->_empty ? () : eval { $self->_format_rdata };
 	carp $@ if $@;
@@ -719,7 +724,7 @@ my %warned;
 
 sub _deprecate {
 	my ( undef, @note ) = @_;
-	carp join ' ', 'deprecated method;', "@note" unless $warned{"@note"}++;
+	carp "deprecated method; @note" unless $warned{"@note"}++;
 	return;
 }
 
@@ -758,17 +763,22 @@ sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
 ## no critic
 sub AUTOLOAD {				## Default method
-	my $self = shift;
-	our $AUTOLOAD;						# keep 'use strict' happy
-	my ($method) = reverse split /::/, $AUTOLOAD;
-
-	for ($method) {			## tolerate mixed-case attribute name
-		return $self->$_(@_) if tr [A-Z-] [a-z_];
-	}
+	my ($self) = @_;
 
 	no strict 'refs';		## no critic ProhibitNoStrict
-	*{$AUTOLOAD} = sub {undef};	## suppress repetition and deep recursion
+	our $AUTOLOAD;
+	my ($method) = reverse split /::/, $AUTOLOAD;
+
+	for ( my $action = $method ) {	## tolerate mixed-case attribute name
+		tr [A-Z-] [a-z_];
+		if ( $self->can($action) ) {
+			*{$AUTOLOAD} = sub { shift->$action(@_) };
+			return &$AUTOLOAD;
+		}
+	}
+
 	my $oref = ref($self);
+	*{$AUTOLOAD} = sub { };		## suppress deep recursion
 	croak qq[$self has no class method "$method"] unless $oref;
 
 	my $string = $self->string;
@@ -789,7 +799,7 @@ $string
 @object
 $@
 END
-	goto &{'Carp::confess'};
+	goto &Carp::confess;
 }
 
 
@@ -833,8 +843,8 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl> L<Net::DNS>
 L<Net::DNS::Question> L<Net::DNS::Packet> L<Net::DNS::Update>
-L<RFC1035(4.1.3)|https://tools.ietf.org/html/rfc1035>
-L<RFC3597|https://tools.ietf.org/html/rfc3597>
+L<RFC1035(4.1.3)|https://iana.org/go/rfc1035#section-4.1.3>
+L<RFC3597|https://iana.org/go/rfc3597>
 
 =cut
 
