@@ -1,75 +1,69 @@
 package Net::DNS::RR::MINFO;
 
-#
-# $Id: MINFO.pm 1382 2015-08-26 20:39:33Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: MINFO.pm 1972 2024-04-21 08:13:19Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::MINFO - DNS MINFO resource record
 
 =cut
 
-
 use integer;
 
 use Net::DNS::Mailbox;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset, @opaque ) = @_;
+	my ( $self, $data, $offset, @opaque ) = @_;
 
-	( $self->{rmailbx}, $offset ) = decode Net::DNS::Mailbox1035(@_);
-	( $self->{emailbx}, $offset ) = decode Net::DNS::Mailbox1035( $data, $offset, @opaque );
+	( $self->{rmailbx}, $offset ) = Net::DNS::Mailbox1035->decode( $data, $offset, @opaque );
+	( $self->{emailbx}, $offset ) = Net::DNS::Mailbox1035->decode( $data, $offset, @opaque );
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
-	my $self = shift;
-	my ( $offset, @opaque ) = @_;
+	my ( $self,   @argument ) = @_;
+	my ( $offset, @opaque )	  = @argument;
 
-	return '' unless $self->{emailbx};
-	my $rdata = $self->{rmailbx}->encode(@_);
+	my $rdata = $self->{rmailbx}->encode(@argument);
 	$rdata .= $self->{emailbx}->encode( $offset + length $rdata, @opaque );
+	return $rdata;
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{emailbx};
-	join ' ', map $self->{$_}->string, qw(rmailbx emailbx);
+	my @rdata = ( $self->{rmailbx}->string, $self->{emailbx}->string );
+	return @rdata;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->rmailbx(shift);
-	$self->emailbx(shift);
+	for (qw(rmailbx emailbx)) { $self->$_( shift @argument ) }
+	return;
 }
 
 
 sub rmailbx {
-	my $self = shift;
-
-	$self->{rmailbx} = new Net::DNS::Mailbox1035(shift) if scalar @_;
-	$self->{rmailbx}->address if defined wantarray && $self->{rmailbx};
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{rmailbx} = Net::DNS::Mailbox1035->new($_) }
+	return $self->{rmailbx} ? $self->{rmailbx}->address : undef;
 }
 
 
 sub emailbx {
-	my $self = shift;
-
-	$self->{emailbx} = new Net::DNS::Mailbox1035(shift) if scalar @_;
-	$self->{emailbx}->address if defined wantarray && $self->{emailbx};
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{emailbx} = Net::DNS::Mailbox1035->new($_) }
+	return $self->{emailbx} ? $self->{emailbx}->address : undef;
 }
 
 
@@ -80,7 +74,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name MINFO rmailbx emailbx');
+    $rr = Net::DNS::RR('name MINFO rmailbx emailbx');
 
 =head1 DESCRIPTION
 
@@ -135,7 +129,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -152,6 +146,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC1035 Section 3.3.7
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC1035(3.3.7)|https://iana.org/go/rfc1035#section-3.3.7>
 
 =cut
