@@ -1,60 +1,57 @@
 package Net::DNS::RR::MR;
 
-#
-# $Id: MR.pm 1382 2015-08-26 20:39:33Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: MR.pm 1972 2024-04-21 08:13:19Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::MR - DNS MR resource record
 
 =cut
 
-
 use integer;
 
 use Net::DNS::DomainName;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->{newname} = decode Net::DNS::DomainName1035(@_);
+	$self->{newname} = Net::DNS::DomainName1035->decode(@argument);
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->{newname}->encode(@_);
+	return $self->{newname}->encode(@argument);
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	$self->{newname}->string;
+	return $self->{newname}->string;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->newname(shift);
+	$self->newname(@argument);
+	return;
 }
 
 
 sub newname {
-	my $self = shift;
-
-	$self->{newname} = new Net::DNS::DomainName1035(shift) if scalar @_;
-	$self->{newname}->name if defined wantarray && $self->{newname};
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{newname} = Net::DNS::DomainName1035->new($_) }
+	return $self->{newname} ? $self->{newname}->name : undef;
 }
 
 
@@ -65,7 +62,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name MR newname');
+    $rr = Net::DNS::RR('name MR newname');
 
 =head1 DESCRIPTION
 
@@ -103,7 +100,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -120,6 +117,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC1035 Section 3.3.8
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC1035(3.3.8)|https://iana.org/go/rfc1035#section-3.3.8>
 
 =cut
