@@ -1,77 +1,67 @@
 package Net::DNS::RR::NID;
 
-#
-# $Id: NID.pm 1408 2015-10-06 20:35:56Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1408 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: NID.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::NID - DNS NID resource record
 
 =cut
 
-
 use integer;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
 	@{$self}{qw(preference nodeid)} = unpack "\@$offset n a8", $$data;
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless defined $self->{nodeid};
-	pack 'n a8', $self->{preference}, $self->{nodeid};
+	return pack 'n a8', $self->{preference}, $self->{nodeid};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless defined $self->{nodeid};
 	return join ' ', $self->preference, $self->nodeid;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->preference(shift);
-	$self->nodeid(shift);
+	for (qw(preference nodeid)) { $self->$_( shift @argument ) }
+	return;
 }
 
 
 sub preference {
-	my $self = shift;
-
-	$self->{preference} = 0 + shift if scalar @_;
-	$self->{preference} || 0;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{preference} = 0 + $_ }
+	return $self->{preference} || 0;
 }
 
 
 sub nodeid {
-	my $self = shift;
-	my $idnt = shift;
-
-	$self->{nodeid} = pack 'n4', map hex($_), split /:/, $idnt if defined $idnt;
-
-	sprintf '%0.4x:%0.4x:%0.4x:%0.4x', unpack 'n4', $self->{nodeid} if $self->{nodeid};
+	my ( $self, $idnt ) = @_;
+	$self->{nodeid} = pack 'n4', map { hex($_) } split /:/, $idnt if defined $idnt;
+	return $self->{nodeid} ? sprintf( '%0.4x:%0.4x:%0.4x:%0.4x', unpack 'n4', $self->{nodeid} ) : undef;
 }
 
 
 my $function = sub {			## sort RRs in numerically ascending order.
-	$Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
+	return $Net::DNS::a->{'preference'} <=> $Net::DNS::b->{'preference'};
 };
 
 __PACKAGE__->set_rrsort_func( 'preference', $function );
@@ -86,9 +76,9 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name IN NID preference nodeid');
+    $rr = Net::DNS::RR->new('name IN NID preference nodeid');
 
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	name	   => 'example.com',
 	type	   => 'NID',
 	preference => 10,
@@ -145,7 +135,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -162,6 +152,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC6742
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC6742|https://tools.ietf.org/html/rfc6742>
 
 =cut
