@@ -1,22 +1,18 @@
 package Net::DNS::RR::NSEC3;
 
-#
-# $Id: NSEC3.pm 1692 2018-07-06 08:55:39Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1692 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: NSEC3.pm 1972 2024-04-21 08:13:19Z willem $)[2];
+
 use base qw(Net::DNS::RR::NSEC);
 
+
 =head1 NAME
 
 Net::DNS::RR::NSEC3 - DNS NSEC3 resource record
 
 =cut
 
-
 use integer;
 
 use base qw(Exporter);
@@ -26,51 +22,23 @@ use Carp;
 
 require Net::DNS::DomainName;
 
-eval 'require Digest::SHA';		## optional for simple Net::DNS RR
-
-my %digest = (
-	'1' => ['Digest::SHA', 1],				# RFC3658
-	);
-
-{
-	my @digestbyname = (
-		'SHA-1' => 1,					# RFC3658
-		);
-
-	my @digestalias = ( 'SHA' => 1 );
-
-	my %digestbyval = reverse @digestbyname;
-
-	my @digestrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @digestbyname;
-	my %digestbyname = ( @digestalias, @digestrehash );	# work around broken cperl
-
-	sub _digestbyname {
-		my $name = shift;
-		my $key	 = uc $name;				# synthetic key
-		$key =~ s /[\W_]//g;				# strip non-alphanumerics
-		$digestbyname{$key} || croak "unknown digest type $name";
-	}
-
-	sub _digestbyval {
-		my $value = shift;
-		$digestbyval{$value} || return $value;
-	}
-}
+eval { require Digest::SHA };		## optional for simple Net::DNS RR
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
 	my $ssize = unpack "\@$offset x4 C", $$data;
-	@{$self}{qw(algorithm flags iterations saltbin)} = unpack "\@$offset CCnx a$ssize", $$data;
+	my ( $algorithm, $flags, $iterations, $saltbin ) = unpack "\@$offset CCnx a$ssize", $$data;
+	@{$self}{qw(algorithm flags iterations saltbin)} = ( $algorithm, $flags, $iterations, $saltbin );
 	$offset += 5 + $ssize;
 	my $hsize = unpack "\@$offset C", $$data;
 	$self->{hnxtname} = unpack "\@$offset x a$hsize", $$data;
 	$offset += 1 + $hsize;
 	$self->{typebm} = substr $$data, $offset, ( $limit - $offset );
-	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
+	$self->{hashfn} = _hashfn( $algorithm, $iterations, $saltbin );
+	return;
 }
 
 
@@ -79,7 +47,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 
 	my $salt = $self->saltbin;
 	my $hash = $self->{hnxtname};
-	pack 'CCn C a* C a* a*', $self->algorithm, $self->flags, $self->iterations,
+	return pack 'CCn C a* C a* a*', $self->algorithm, $self->flags, $self->iterations,
 			length($salt), $salt,
 			length($hash), $hash,
 			$self->{typebm};
@@ -90,23 +58,25 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my @rdata = (
-		$self->algorithm, $self->flags, $self->iterations,
+		$self->algorithm,   $self->flags,    $self->iterations,
 		$self->salt || '-', $self->hnxtname, $self->typelist
 		);
+	return @rdata;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->algorithm(shift);
-	$self->flags(shift);
-	$self->iterations(shift);
-	my $salt = shift;
+	my $alg = $self->algorithm( shift @argument );
+	$self->flags( shift @argument );
+	my $iter = $self->iterations( shift @argument );
+	my $salt = shift @argument;
 	$self->salt($salt) unless $salt eq '-';
-	$self->hnxtname(shift);
-	$self->typelist(@_);
-	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
+	$self->hnxtname( shift @argument );
+	$self->typelist(@argument);
+	$self->{hashfn} = _hashfn( $alg, $iter, $self->{saltbin} );
+	return;
 }
 
 
@@ -114,6 +84,7 @@ sub _defaults {				## specify RR attribute default values
 	my $self = shift;
 
 	$self->_parse_rdata( 1, 0, 0, '' );
+	return;
 }
 
 
@@ -132,61 +103,71 @@ sub algorithm {
 
 
 sub flags {
-	my $self = shift;
-
-	$self->{flags} = 0 + shift if scalar @_;
-	$self->{flags} || 0;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{flags} = 0 + $_ }
+	return $self->{flags} || 0;
 }
 
 
 sub optout {
-	my $bit = 0x01;
-	for ( shift->{flags} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
-		return $_ & $bit;
+	my ( $self, @value ) = @_;
+	for ( $self->{flags} |= 0 ) {
+		if ( scalar @value ) {
+			$_ |= 0x01;
+			$_ ^= 0x01 unless shift @value;
+		}
 	}
+	return $self->{flags} & 0x01;
 }
 
 
 sub iterations {
-	my $self = shift;
-
-	$self->{iterations} = 0 + shift if scalar @_;
-	$self->{iterations} || 0;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{iterations} = 0 + $_ }
+	return $self->{iterations} || 0;
 }
 
 
 sub salt {
-	my $self = shift;
-	return unpack "H*", $self->saltbin() unless scalar @_;
-	$self->saltbin( pack "H*", map /[^\dA-F]/i ? croak "corrupt hex" : $_, join "", @_ );
+	my ( $self, @value ) = @_;
+	return unpack "H*", $self->saltbin() unless scalar @value;
+	my @hex = map { /^"*([\dA-Fa-f]*)"*$/ || croak("corrupt hex"); $1 } @value;
+	return $self->saltbin( pack "H*", join "", @hex );
 }
 
 
 sub saltbin {
-	my $self = shift;
-
-	$self->{saltbin} = shift if scalar @_;
-	$self->{saltbin} || "";
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{saltbin} = $_ }
+	return $self->{saltbin} || "";
 }
 
 
 sub hnxtname {
-	my $self = shift;
-	$self->{hnxtname} = _decode_base32hex(shift) if scalar @_;
-	_encode_base32hex( $self->{hnxtname} ) if defined wantarray;
+	my ( $self, @name ) = @_;
+	for (@name) { $self->{hnxtname} = _decode_base32hex($_) }
+	return defined(wantarray) ? _encode_base32hex( $self->{hnxtname} ) : undef;
 }
 
 
+sub match {
+	my ( $self, $name ) = @_;
+
+	my ($owner) = $self->{owner}->label;
+	my $ownerhash = _decode_base32hex($owner);
+
+	my $hashfn = $self->{hashfn};
+	return $ownerhash eq &$hashfn($name);
+}
+
 sub covers {
 	my ( $self, $name ) = @_;
 
-	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my ( $owner, @zone ) = $self->{owner}->label;
 	my $ownerhash = _decode_base32hex($owner);
 	my $nexthash  = $self->{hnxtname};
 
-	my @label = new Net::DNS::DomainName($name)->_wire;
+	my @label = Net::DNS::DomainName->new($name)->label;
 	my @close = @label;
 	foreach (@zone) { pop(@close) }				# strip zone labels
 	return if lc($name) ne lc( join '.', @close, @zone );	# out of zone
@@ -204,29 +185,14 @@ sub covers {
 }
 
 
-sub covered {				## historical
-	&covers;						# uncoverable pod
-}
-
-sub match {				## historical
-	my ( $self, $name ) = @_;				# uncoverable pod
-
-	my ($owner) = $self->{owner}->_wire;
-	my $ownerhash = _decode_base32hex($owner);
-
-	my $hashfn = $self->{hashfn};
-	$ownerhash eq &$hashfn($name);
-}
-
-
 sub encloser {
 	my ( $self, $qname ) = @_;
 
-	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my ( $owner, @zone ) = $self->{owner}->label;
 	my $ownerhash = _decode_base32hex($owner);
 	my $nexthash  = $self->{hnxtname};
 
-	my @label = new Net::DNS::DomainName($qname)->_wire;
+	my @label = Net::DNS::DomainName->new($qname)->label;
 	my @close = @label;
 	foreach (@zone) { pop(@close) }				# strip zone labels
 	return if lc($qname) ne lc( join '.', @close, @zone );	# out of zone
@@ -234,14 +200,13 @@ sub encloser {
 	my $hashfn = $self->{hashfn};
 
 	my $encloser = $qname;
-	shift @label;
 	foreach (@close) {
 		my $nextcloser = $encloser;
-		my $hash = &$hashfn( $encloser = join '.', @label );
 		shift @label;
+		my $hash = &$hashfn( $encloser = join '.', @label );
 		next if $hash ne $ownerhash;
 		$self->{nextcloser} = $nextcloser;		# next closer name
-		$self->{wildcard} = join '.', '*', $encloser;	# wildcard at provable encloser
+		$self->{wildcard}   = "*.$encloser";		# wildcard at provable encloser
 		return $encloser;				# provable encloser
 	}
 	return;
@@ -255,17 +220,49 @@ sub wildcard { return shift->{wildcard}; }
 
 ########################################
 
+my @digestbyname = (
+	'SHA-1' => 1,						# [RFC3658]
+	);
+
+my @digestalias = ( 'SHA' => 1 );
+
+my %digestbyval = reverse @digestbyname;
+
+foreach (@digestbyname) { s/[\W_]//g; }				# strip non-alphanumerics
+my @digestrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @digestbyname;
+my %digestbyname = ( @digestalias, @digestrehash );		# work around broken cperl
+
+sub _digestbyname {
+	my $arg = shift;
+	my $key = uc $arg;					# synthetic key
+	$key =~ s/[\W_]//g;					# strip non-alphanumerics
+	my $val = $digestbyname{$key};
+	croak qq[unknown algorithm $arg] unless defined $val;
+	return $val;
+}
+
+sub _digestbyval {
+	my $value = shift;
+	return $digestbyval{$value} || return $value;
+}
+
+
+my %digest = (
+	'1' => scalar( eval { Digest::SHA->new(1) } ),		# RFC3658
+	);
+
+
 sub _decode_base32hex {
 	local $_ = shift || '';
 	tr [0-9A-Va-v\060-\071\101-\126\141-\166] [\000-\037\012-\037\000-\037\012-\037];
 	my $l = ( 5 * length ) & ~7;
-	pack "B$l", join '', map unpack( 'x3a5', unpack 'B8', $_ ), split //;
+	return pack "B$l", join '', map { unpack( 'x3a5', unpack 'B8', $_ ) } split //;
 }
 
 
 sub _encode_base32hex {
-	my @split = grep length, split /(\S{5})/, unpack 'B*', shift;
-	local $_ = join '', map pack( 'B*', "000$_" ), @split;
+	my @split = grep {length} split /(\S{5})/, unpack 'B*', shift;
+	local $_ = join '', map { pack( 'B*', "000$_" ) } @split;
 	tr [\000-\037] [0-9a-v];
 	return $_;
 }
@@ -275,37 +272,38 @@ my ( $cache1, $cache2, $limit ) = ( {}, {}, 10 );
 
 sub _hashfn {
 	my $hashalg    = shift;
-	my $iterations = shift;
+	my $iterations = shift || 0;
 	my $salt       = shift || '';
 
-	my $arglist = $digest{$hashalg};
-	my ( $class, @argument ) = @$arglist;
-	my $instance = $class->new(@argument);
+	my $hash = $digest{$hashalg};
+	return sub { croak "algorithm $hashalg not supported" }
+			unless $hash;
+	my $clone = $hash->clone;
 
-	$iterations++;
 	my $key_adjunct = pack 'Cna*', $hashalg, $iterations, $salt;
 
 	return sub {
-		my $name  = new Net::DNS::DomainName(shift)->canonical;
+		my $name  = Net::DNS::DomainName->new(shift)->canonical;
 		my $key	  = join '', $name, $key_adjunct;
 		my $cache = $$cache1{$key} ||= $$cache2{$key};	# two layer cache
 		return $cache if defined $cache;
 		( $cache1, $cache2, $limit ) = ( {}, $cache1, 50 ) unless $limit--;    # recycle cache
 
-		my $hash = $name;
-		my $iter = $iterations;
-		$instance->reset;
-		while ( $iter-- ) {
-			$instance->add($hash);
-			$instance->add($salt);
-			$hash = $instance->digest;
+		$clone->add($name);
+		$clone->add($salt);
+		my $digest = $clone->digest;
+		my $count  = $iterations;
+		while ( $count-- ) {
+			$clone->add($digest);
+			$clone->add($salt);
+			$digest = $clone->digest;
 		}
-		return $$cache1{$key} = $hash;
+		return $$cache1{$key} = $digest;
 	};
 }
 
 
-sub hashalgo { &algorithm; }					# uncoverable pod
+sub hashalgo { return &algorithm; }				# uncoverable pod
 
 sub name2hash {
 	my $hashalg    = shift;					# uncoverable pod
@@ -313,9 +311,11 @@ sub name2hash {
 	my $iterations = shift || 0;
 	my $salt       = pack 'H*', shift || '';
 	my $hash       = _hashfn( $hashalg, $iterations, $salt );
-	_encode_base32hex( &$hash($name) );
+	return _encode_base32hex( &$hash($name) );
 }
 
+########################################
+
 
 1;
 __END__
@@ -324,7 +324,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name NSEC3 algorithm flags iterations salt hnxtname');
+    $rr = Net::DNS::RR->new('name NSEC3 algorithm flags iterations salt hnxtname');
 
 =head1 DESCRIPTION
 
@@ -431,6 +431,13 @@ interpolated into a string.
 typemap() returns a Boolean true value if the specified RRtype occurs
 in the type bitmap of the NSEC3 record.
 
+=head2 match
+
+    $matched = $rr->match( 'example.foo' );
+
+match() returns a Boolean true value if the hash of the domain name
+argument matches the hashed owner name of the NSEC3 RR.
+
 =head2 covers
 
     $covered = $rr->covers( 'example.foo' );
@@ -471,7 +478,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -488,8 +495,10 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC5155, RFC4648
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC5155(3)|https://iana.org/go/rfc5155#section-3>
+L<RFC9077|https://iana.org/go/rfc9077>
 
-L<Hash Algorithms|http://www.iana.org/assignments/dnssec-nsec3-parameters>
+L<Hash Algorithms|https://iana.org/assignments/dnssec-nsec3-parameters>
 
 =cut
