@@ -1,94 +1,88 @@
 package Net::DNS::RR::URI;
 
-#
-# $Id: URI.pm 1356 2015-06-11 08:44:12Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1356 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: URI.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::URI - DNS URI resource record
 
 =cut
 
-
 use integer;
 
 use Net::DNS::Text;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+sub _decode_rdata {			## decode rdata from wire-format octet string
+	my ( $self, $data, $offset ) = @_;
 
 	my $limit = $offset + $self->{rdlength};
 	@{$self}{qw(priority weight)} = unpack( "\@$offset n2", $$data );
 	$offset += 4;
-	$self->{target} = decode Net::DNS::Text( $data, $offset, $limit - $offset );
+	$self->{target} = Net::DNS::Text->decode( $data, $offset, $limit - $offset );
+	return;
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $target = $self->{target} || return '';
-	pack 'n2 a*', @{$self}{qw(priority weight)}, $target->raw;
+	my $target = $self->{target};
+	return pack 'n2 a*', @{$self}{qw(priority weight)}, $target->raw;
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $target = $self->{target} || return '';
-	my @rdata = ( $self->priority, $self->weight, $target->string );
+	my $target = $self->{target};
+	my @rdata  = ( $self->priority, $self->weight, $target->string );
+	return @rdata;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+sub _parse_rdata {			## populate RR from rdata in argument list
+	my ( $self, @argument ) = @_;
 
-	map $self->$_(shift), qw(priority weight target);
+	for (qw(priority weight target)) { $self->$_( shift @argument ) }
+	return;
 }
 
 
 sub priority {
-	my $self = shift;
-
-	$self->{priority} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{priority} = 0 + $_ }
 	return $self->{priority} || 0;
 }
 
 
 sub weight {
-	my $self = shift;
-
-	$self->{weight} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{weight} = 0 + $_ }
 	return $self->{weight} || 0;
 }
 
 
 sub target {
-	my $self = shift;
-
-	$self->{target} = new Net::DNS::Text(shift) if scalar @_;
-	$self->{target}->value if defined wantarray && $self->{target};
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{target} = Net::DNS::Text->new($_) }
+	return $self->{target} ? $self->{target}->value : undef;
 }
 
 
 # order RRs by numerically increasing priority, decreasing weight
-my $function = __PACKAGE__->set_rrsort_func(
-	'priority',
-	sub {
-		my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
-		$a->{priority} <=> $b->{priority}
-				|| $b->{weight} <=> $a->{weight};
-	} );
+my $function = sub {
+	my ( $a, $b ) = ( $Net::DNS::a, $Net::DNS::b );
+	return $a->{priority} <=> $b->{priority}
+			|| $b->{weight} <=> $a->{weight};
+};
 
+__PACKAGE__->set_rrsort_func( 'priority', $function );
 
 __PACKAGE__->set_rrsort_func( 'default_sort', $function );
 
@@ -100,7 +94,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name URI priority weight target');
+    $rr = Net::DNS::RR->new('name URI priority weight target');
 
 =head1 DESCRIPTION
 
@@ -159,7 +153,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -176,7 +170,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, 
-RFC7553
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC7553|https://tools.ietf.org/html/rfc7553>
 
 =cut
