@@ -1,9 +1,8 @@
 package Net::DNS::Resolver::UNIX;
 
-#
-# $Id: UNIX.pm 1568 2017-05-27 06:40:20Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1568 $)[1];
+use strict;
+use warnings;
+our $VERSION = (qw$Id: UNIX.pm 1981 2024-06-17 13:22:14Z willem $)[2];
 
 
 =head1 NAME
@@ -13,28 +12,32 @@ Net::DNS::Resolver::UNIX - Unix resolver class
 =cut
 
 
-use strict;
-use warnings;
-use base qw(Net::DNS::Resolver::Base);
-
-
-my @config_file = grep -f $_ && -r _, '/etc/resolv.conf';
+my @config_file = grep { -f $_ && -r _ } '/etc/resolv.conf';
 
+my $homedir = $ENV{HOME};
 my $dotfile = '.resolv.conf';
-my @dotpath = grep defined, $ENV{HOME}, '.';
-my @dotfile = grep -f $_ && -o _, map "$_/$dotfile", @dotpath;
+my @dotfile = grep { -f $_ && -o _ } map {"$_/$dotfile"} grep {$_} $homedir, '.';
+
+
+my $path = $ENV{PATH};
+local $ENV{PATH} = join ':', grep {$_} qw(/bin /usr/bin), $path;
+my $uname = eval {`uname -n 2>/dev/null`} || '';
+chomp $uname;
+my ( $host, @domain ) = split /\./, $uname, 2;
 
 
 sub _init {
 	my $defaults = shift->_defaults;
 
-	map $defaults->_read_config_file($_), @config_file;
+	$defaults->domain(@domain);
+	$defaults->_read_config_file($_) foreach @config_file;
 
 	%$defaults = Net::DNS::Resolver::Base::_untaint(%$defaults);
 
-	map $defaults->_read_config_file($_), @dotfile;
+	$defaults->_read_config_file($_) foreach @dotfile;
 
 	$defaults->_read_env;
+	return;
 }
 
 
@@ -63,7 +66,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
